@@ -11,6 +11,7 @@ import os
 import sys
 import json
 import traceback
+import fnmatch
 
 from . import __version__ as VERSION
 from .wandbox import Wandbox
@@ -32,27 +33,28 @@ class CLI:
         self.setup(lang, compiler, has_option)
 
     def command_list(self, args):
-        r = get_compiler_list(args.retry, args.retry_wait)
+        r = self.get_compiler_list(args.retry, args.retry_wait)
         if args.language:
             r = [x for x in r if args.language == x['language']]
         if args.compiler:
-            r = [x for x in r if args.compiler == x['name']]
+            r = [x for x in r if fnmatch.fnmatch(x['name'], args.compiler)]
         print(json.dumps(r, indent=2))
 
     def command_lang(self, args):
         if args.language:
             print(args.language)
         else:
-            r = get_compiler_list(args.retry, args.retry_wait)
+            r = self.get_compiler_list(args.retry, args.retry_wait)
             langs = map(lambda x: x['language'], r)
             print('\n'.join(sorted(set(langs))))
 
     def command_compiler(self, args):
-        r = get_compiler_list(args.retry, args.retry_wait)
+        r = self.get_compiler_list(args.retry, args.retry_wait)
         for d in r:
             if args.language:
                 if args.language == d['language']:
-                    print(d['name'])
+                    if (args.compiler is None) or (fnmatch.fnmatch(d['name'], args.compiler)):
+                        print(d['name'])
             else:
                 print('{0}: {1}'.format(d['language'], d['name']))
 
@@ -63,7 +65,7 @@ class CLI:
         return '{0}{1} (default)'.format(' ' * indent, name)
 
     def command_options(self, args):
-        r = get_compiler_list(args.retry, args.retry_wait)
+        r = self.get_compiler_list(args.retry, args.retry_wait)
         for d in r:
             prefix = ''
             indent = 0
@@ -73,7 +75,7 @@ class CLI:
             else:
                 prefix = '{0}: '.format(d['language'])
             if args.compiler:
-                if args.compiler != d['name']:
+                if not fnmatch.fnmatch(d['name'], args.compiler):
                     continue
             else:
                 prefix = '{0}: '.format(d['name'])
@@ -107,6 +109,19 @@ class CLI:
         b = Wandbox.ShowResult(r['result'])
         sys.exit(b)
 
+    def command_template(self, args):
+        print(self.get_template_code(args))
+
+    def command_run_template(self, args):
+        setattr(args, 'sources', ['-'])
+        sys.stdin = self.get_template_code(args)
+        options = []
+        for o in args.options:
+            options.extend(o.split(','))
+        runner = self.get_runner(args, options)
+        self.setup_runner(args, options, [], runner)
+        self.run_with_runner(args, runner)
+
     def command_run(self, args):
         options = []
         for o in args.options:
@@ -140,15 +155,25 @@ class CLI:
     def get_compiler_list(self, retry, wait):
         return Wandbox.Call(Wandbox.GetCompilerList, retry, wait)
 
+    def get_template(self, name, retry, wait):
+        return Wandbox.Call(lambda : Wandbox.GetTemplate(name), retry, wait)
+
     def auto_setup_compiler(self, args):
-        if args.language and args.compiler is None:
-            r = self.get_compiler_list(args.retry, args.retry_wait)
-            for d in r:
-                if args.language == d['language']:
-                    if args.no_head and 'head' in d['name']:
-                        continue
-                    args.compiler = d['name']
-                    break
+        cond = '*'
+        if args.compiler:
+            if fnmatch.translate(args.compiler) == args.compiler:
+                return
+            cond = args.compiler
+        r = self.get_compiler_list(args.retry, args.retry_wait)
+        for d in r:
+            if args.language and args.language != d['language']:
+                continue
+            if args.no_head and 'head' in d['name']:
+                continue
+            if fnmatch.fnmatch(d['name'], cond):
+                args.language = d['language']
+                args.compiler = d['name']
+                break
 
     def run_with_runner(self, args, runner):
         if args.dryrun:
@@ -188,7 +213,8 @@ class CLI:
             '-c',
             '--compiler',
             default=compiler,
-            help=SUPPRESS if compiler else 'specify compiler'
+            help=SUPPRESS if compiler else
+            'specify compiler (can use fnmatch, use first match compiler. e.g. clang-3.9.*[!c] => clang-3.9.1)'
         )
         self.parser.add_argument(
             '-x',
@@ -262,14 +288,18 @@ class CLI:
         compiler_cmd = subparser.add_parser(
             'compiler',
             description='show support compilers',
-            help='show support compilers. see `compiler -h`')
-        compiler_cmd.set_defaults(handler=self.command_compiler)
-
-        compiler_cmd = subparser.add_parser(
+            help=SUPPRESS)
+        compilers_cmd = subparser.add_parser(
+            'compilers',
+            description='show support compilers',
+            help='show support compilers. see `compilers -h`')
+        versions_cmd = subparser.add_parser(
             'versions',
             description='show support compilers',
             help='show support compilers. see `versions -h`')
-        compiler_cmd.set_defaults(handler=self.command_compiler)
+        compilers_cmds = [compiler_cmd, compilers_cmd, versions_cmd]
+        for compiler_cmd_ in compilers_cmds:
+            compiler_cmd_.set_defaults(handler=self.command_compiler)
 
         lang_cmd = subparser.add_parser(
             'lang',
@@ -325,6 +355,27 @@ class CLI:
                 help='comiple options'
             )
 
+        template_cmd = subparser.add_parser(
+            'template',
+            description='get template code',
+            help='get wandbox template code. see `template -h`'
+        )
+        template_cmd.set_defaults(handler=self.command_template)
+
+        run_template_cmd = subparser.add_parser(
+            'run-template',
+            prefix_chars='+',
+            description='run wandbox template code',
+            help='run wandbox template code. see `run-template +h`'
+        )
+        run_template_cmd.set_defaults(handler=self.command_run_template)
+        run_template_cmd.add_argument(
+            'compile_options',
+            metavar='COMPILE_OPTIONS',
+            nargs='*',
+            help='comiple options'
+        )
+
         subcommands = self.parser.format_usage().split('{')[1].split('}')[0]
         help_cmd = subparser.add_parser('help', help='show subcommand help. see `help -h`')
         help_cmd.set_defaults(handler=self.command_help)
@@ -368,3 +419,25 @@ class CLI:
         if hasattr(args, attr_name):
             if getattr(args, attr_name):
                 options.append(opt_name)
+
+    def find_compilers(self, list_json, args):
+        find = []
+        for d in list_json:
+            if args.language:
+                if args.language == d['language']:
+                    if (args.compiler is None) or (fnmatch.fnmatch(d['name'], args.compiler)):
+                        find.append(d)
+            else:
+                find.append(d)
+        return find
+
+    def get_template_code(self, args):
+        r = self.get_compiler_list(args.retry, args.retry_wait)
+        setattr(args, 'sources', ['-'])
+        self.auto_setup_compiler(args)
+        compiler = self.find_compilers(r, args)
+        if len(compiler) != 1:
+            raise Exception('Detected multiple compilers. Please specify so that it becomes one.')
+        template_name = compiler[0]['templates'][0]
+        template = self.get_template(template_name, args.retry, args.retry_wait)
+        return template['code']
